#!/usr/bin/perl 
#
#	Program:	hwpmon.pl  -   HotWire pipe monitor
#	Purpose:	Monitors a named pipe looking for fax requests from 
#				UNIX, PCs, MACs or OS/2
#	Date:		01-21-1997
#	Author:		Gary C. Heller
#
use	FileHandle;
use Getopt::Std;

getopts("dhs:");

# hwpmon args -- the required fields
%dparam = (
					 "Dial" => "-",
					 "Subject" => "-",
					 "Coverpage" => "NOCOVER",
					 "From" => "-",
					 "To" => "-",
					 "Priority" => "0",
#					 "Delete After" => "-",
#					 "Print After" => "-",
					 "Modem Port" => "-",
#					 "Resolution" => "-",
#					 "Retries" => "-",
					 "Userid" => "-",
					 "BEGIN REQUEST" => "-",
					 "END REQUEST" => "-",
					 "BEGIN FILE" => "1",
					 "ATTACH" => "1",
					 "END FILE" => "1"
					);

# hwdispatch args
%darg = (
				 "Dial" => "-t",
				 "Subject" => "-Z",
				 "Coverpage" => "-C",
				 "From" => "-F",
				 "To" => "-T",
				 "Priority" => "-p",
				 "Delete After" => "-D",
				 "Print After" => "-R",
				 "Modem Port" => "-d",
				 "Resolution" => "-r",
				 "Retries" => "-m",
				 "Userid" => "-U"
				);

# Ignore list
%ignore = (
					 "BEGIN REQUEST" => "1",
					 "END REQUEST" => "1",
					 "BEGIN FILE" => "1",
					 "END FILE" => "1"
					);

$debug = "";
$request_pipe = "";
$hwroot = "";
$hwtmp = "";
$hw_license = "/etc/HotWire";
$resilient = 0;									# Keep going even if errors
@attachments = ();							# List of filenames that are attachments

if ($opt_d) {
	$debug = "y";
}

$SIG{INT} = \&sig_clean;
$SIG{QUIT} = \&sig_clean;
$SIG{HUP} = \&sig_clean;
$SIG{TERM} = \&sig_clean;

sub	sig_clean()
	{
		if ($debug) {
			print "\nHotWire: Caught terminate signal!\n";
		}
		unlink($request_pipe);
		exit 1;
	}


sub	isroot()
	{
		if ($< != 0 && $> != 0) {
			return 0;
		}
		return 1;
	}


sub	stop_monitor()
	{
		if ($debug) {
			print "STOPPING MONITOR...........\n";
		}
		
		if (! -p $request_pipe ) {
			print "HotWire: Client Request Queue already stopped!\n";
			return;
		}
				
		$qfd = FileHandle->new();
		open($qfd, ">$request_pipe") || die "ERROR: Can't open $request_pipe!";
		print $qfd "QUIT=YES\n";
		print "HotWire: Client Request Queue stopped\n";
		$qfd->close();
		$rc = unlink($request_pipe);		
		return;
	}

sub check_pipe()
	{
		#
		#	Make sure we have a valid pipe
		#
		if ( ! -p $request_pipe ) {
			$ret = system("mknod $request_pipe p");
			if ($ret == 0) {
				chmod(0766, $request_pipe);
			}	else {
				print "ERROR: Can't create named pipe $request_pipe!\n";
				print "ERROR: Have to abort!\n";
				exit 1;
			}
		}	else {
			print "HotWire: Client Request Queue already active!\n";
			exit 1;
		}
	}

sub	init_params()
	{
		if (! -r $hw_license ) {
			print "ERROR: Can't read /etc/HotWire - aborting!\n";
			exit 1;
		}
		$hwfd = FileHandle->new();
		open($hwfd, "< $hw_license") 
			|| die "ERROR: Can't open HotWire license file.\n";
		$hwroot = <$hwfd>;
		chomp($hwroot);
		$hwfd->close();
		
		$hwtmp = $hwroot . "/tmp";
		
		get_configs();
		
		return 1;
	}


sub	get_configs()
	{
		$config_fd = FileHandle->new();

		open($config_fd, "$hwroot/config.ttyS1") || 
			die "ERROR: Can't open HotWire config file";
		while ($line = <$config_fd>) {
			chomp($line);
			if ( $line =~ /^RequestPipe (\S+)/ ) {
				$request_pipe = $1;
				last;
			}
		}
		
		if (! $request_pipe) {
			print "ERROR: No request pipe defined - use Admin/Config-Setup option\n";
			exit 1;
		}
	}

sub	dispatch_request()
	{
		print "REQUEST DISPATCHED!\n\n";
		if ($debug) {
			foreach $att (@attachments) {
				print "FILE: $att\n";
			}
			print "\n";
		}
		#
		#	Make sure we have the BARE MINIMUM we need
		#
		if (!defined $hash{'Coverpage'}) {
			$hash{'Coverpage'} = "NOCOVER";
		}
		if (!defined $hash{'Priority'}) {
			$hash{'Priority'} = "0";
		}
		
		$cmd = "$hwroot/bin/hwdispatch.pl ";
		foreach $parm ( keys %dparam ) {
			if (defined $darg{$parm} ) {
				$cmd .= " $darg{$parm} \'$hash{$parm}\'";
			}
		}

		$cmd .= " ";
		foreach $att (@attachments) {
			$cmd .= $att . " ";
		}

		if ($debug) {
			print "COMMAND=$cmd\n";
		}
		system("$cmd");
		return 1;
	}

sub	get_temp_filename()
	{
		$temp_file = $hwtmp . "/hw_t_f" . $$ . "_" . (@attachments);
		return $temp_file;
	}

#
#	Get a 'file' like a 'here' document in sh, embedded within the request.
#
sub	do_inline_file()
	{
		$found = 0;
		$tfname = get_temp_filename();
		$tfd = FileHandle->new();
		open($tfd, "> $tfname") || die "ERROR: Can't open temp file $tfname!\n";
		while ($fline = <$pfd>) 
			{
				if ( $fline =~ /^END FILE/ ) {
					$found = 1;
					last;
				}
				print $tfd $fline;
			}
		$tfd->close();
		if (!$found)								# Found end of inline file data
			{
				print "ERROR!  End of inline file (END FILE) not found!\n";
				print "ERRIR!  Inline data file ignored!\n";
				unlink($tfname);
			}
		push(@attachments, $tfname);
		return 1;
	}


sub	valid_request()
	{
		$ret = 1;
		
		foreach $iline ( @request ) {
			if ( $iline =~ /QUIT=YES/ ) {	# Cause this collector to quit
				unlink($request_pipe);
				exit 0;
			}
			$val = "";
			($mnemonic, $val) = split('=', $iline);
			if ($mnemonic eq 'To' || $mnemonic eq 'From') {
				$val =~ s/ /_/g;
				if ($debug) {
					print "New Value = $val", "\n";
				}
			}
			$hash{$mnemonic} = $val;
			if (!$dparam{$mnemonic})
				{
					print "ERROR: Invalid parameter '$mnemonic' - ignored!\n";
				}
			else
				{
					$dparam{$mnemonic} = $val;
				}
		}
		#
		#	Make sure that we have the minimum number of arguments
		#
		foreach $required ( keys %dparam )
			{
				if ($ignore{$required}) {
					next;
				}
				if (!defined($dparam{$required}) || $dparam{$required} eq '-' )
					{
						print "Missing parameter '$required' \n";
						$ret = 0;
					}
			}
		return $ret;
	}


####################### MAIN LOGIC #######################
####################### MAIN LOGIC #######################
####################### MAIN LOGIC #######################

init_params();

if (! isroot()) {
	print "HotWire: You don't have permission - must be superuser!\n";
	exit 1;
}

#	if ($opt_s =~ /top/) {
if ($opt_s =~ /stop/) {					# CCC: Changed from "top" to "stop"
	stop_monitor();
	exit 1;
}

check_pipe();

$pfd = FileHandle->new();
while (1) {
	open($pfd, "< $request_pipe") || die "ERROR: Can't open $request_pipe!";
	@request = ();								# Empty array
	@attachments = ();
	%hash = ();										# Empty hash
	$discard = 0;
	#
	#	Collect a request
	#
	while($line = <$pfd>) {
		if ($debug) {
			print "LINE# $. :  $line";
		}
		chomp($line);
		
		if ($. == 1 && $line eq "QUIT=YES") {
			unlink($request_pipe);
			exit 0;
		}
		
		if ($. == 1 && $line ne "BEGIN REQUEST") {
			print "ERROR: No BEGIN REQUEST line - request discarded!\n";
			$discard = 1;
		}
		
		if ($line eq "END REQUEST") { 
			last;
		}
		
		if ($line eq "BEGIN FILE") {
			print "\ndo_inline_file\n";
			do_inline_file();
		}
		elsif ($line =~ /^ATTACH=/) {	# Here's a file to include
			($mnem, $a_file) = split('=', $line, 2);
			print "\npush to attachments\n";
			push(@attachments, $a_file);
		}
		else
			{
				print "\npush to request\n";
				push(@request, $line);
			}
	}
	
	$pfd->close();
	if ($discard) {
		print "REQUEST DISCARDED!\n";
		next;
	}
	#
	#	We have a complete request here, now lets validate it
	#
	if (!valid_request()) {
		print "ERROR: Invalid Request!\n";
		next;
	}
	print "VALID: Request is OK\n";
	#
	#	OK, let's dispatch this request
	#
	if (!dispatch_request()) {
		print "ERROR: Can't dispatch request!\n";
		next;
	}
} ## while-loop

#
#	End of program
#
